//  Beispiel 8 "Berechnung der Fließkomma-Rechenleistung"
//  
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>

float a = 0.0, b = 3.0, Zeit = 0.0;
long Start = 0, Ende = 0, Groesse = 1;

void setup()
{
  IoT_Init(true);
  IoT_TerminalWaitInit(true);      // Auf Terminalverbindung warten, damit nichts verloren geht
  IoT_WatchDog(false);
  Serial.println("");
  Serial.print(chr(12));
  Serial.println("Welcome to IoT Brick GFLOP Test" + spc(35) + "(c) 2015-2017");
  Serial.println(repeatstr("-", 79) + chr(13));
  Serial.print("CPU " + t_CPUname());
  Serial.print(" (ID " + str(t_CPUtype()) + "), ");
  Serial.print(str(t_CPUbits()) + " Bit, ");
  Serial.print(str(t_CPUflashRAM()) + " kB Flash-RAM, ");
  Serial.println(str(t_CPUstaticRAM()) + " kB SRAM" + char(13));
  t_DateTime DateTime = t_CompileDateTime();        // Uhrzeit, wann der Sketch kompiliert wurde
  byte dayOfWeek = dayofweek(DateTime.year, DateTime.month, DateTime.day);
  Serial.println("Date : " + datestr(DateTime)+" (" + dayname(dayOfWeek)+")");
  Serial.println("Time : " + timestr(DateTime));
  Serial.println("");//....1....!....2....!....3....!....4....!....5....!....6....!....7....!....8
  Serial.println("Der GFLOP Test berechnet die Anzahl an Fließkomma-Additionen, die der IoT-Brick");
  Serial.println("pro Sekunde berechnen kann. Mit jeder Zeile werden mehr Additionen berechnet,");
  Serial.println("wodurch eine höhere Genauigkeit erreicht wird. Das Maximum sind 10 Mio. Fließ-");
  Serial.println("komma-Additionen. Der IoT-Brick berechnet ca. 1,4 Mio. Additionen pro Sekunde.");
  Serial.println("Arduino Nano, Uno oder Mega 2560 berechnen ca. 0,1 Mio. Additionen pro Sekunde.");
  Serial.println("");
  while (Zeit < 5000000)
  {
    Groesse = Groesse * 10;
    Start = micros();
    a = 0.0;
    for (long i = 1; i <= Groesse; i++)
    {
        a = a + b;
    }
    Ende = micros();
    Zeit = Ende - Start;
    if (Zeit > 0)
    {
      Serial.print(strform(Groesse, 32, 1, true));
      Serial.print(" Schritte - benötigte Zeit = ");
      Serial.print(strform(Zeit / 1000, 32, 1, true));
      Serial.print(" ms. = ");
      Serial.print(strform(Groesse * 1000.0 / Zeit * 1000.0, 32, 1, true));
      Serial.println(" FLOPs.");
      delay(10);
    }
  }
  String S = String(a); // Verhindert, dass die for-Schleife wegoptimiert wird
  Serial.println("");
  Serial.println("Fertig.");
}

void loop()
{
  IoT_ShutDown();
}

// Arduino Nano ( 16 MHz,  8 Bit) =   104.024 FLOPs ( 1 Mio. Schritte)
// Arduino Mega ( 16 MHz,  8 Bit) =   100.717 FLOPs ( 1 Mio. Schritte)
// Arduino Due  ( 84 MHz, 32 Bit) =   822.665 FLOPs (10 Mio. Schritte)
// Teensy 3.1   ( 96 MHz, 32 Bit) = 1.408.183 FLOPs (10 Mio. Schritte)
// Teensy 3.1   (120 MHz, 32 Bit) = 1.709.976 FLOPs (10 Mio. Schritte)
// Teensy 3.1   (144 MHz, 32 Bit) = 1.690.433 FLOPs (10 Mio. Schritte)
// ESP 8266     ( 80 MHz, 32 Bit) = 1.428.530 FLOPs ( 1 Mio. Schritte)

